@@ -1,4 +1,4 @@
-/*
+/* vi:set et sw=2 sts=2 cin cino=t0,f0,(0,{s,>2s,n-s,^-s,e-s:
  * Copyright © 2018 Red Hat, Inc
  *
  * This program is free software; you can redistribute it and/or
@@ -28,7 +28,7 @@
 
 #include <glib/gi18n.h>
 
-#include "libglnx/libglnx.h"
+#include "libglnx.h"
 #include "flatpak-permission-dbus-generated.h"
 
 #include "flatpak-builtins.h"
@@ -133,7 +133,7 @@ flatpak_builtin_permission_list (int argc, char **argv,
   XdpDbusPermissionStore *store = NULL;
   const char *table;
   const char *id;
-  FlatpakTablePrinter *printer = NULL;
+  g_autoptr(FlatpakTablePrinter) printer = NULL;
 
   context = g_option_context_new (_("[TABLE] [ID] - List permissions"));
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
@@ -184,7 +184,6 @@ flatpak_builtin_permission_list (int argc, char **argv,
     }
   else
     {
-      int i;
       g_auto(GStrv) tables = get_permission_tables (store);
 
       for (i = 0; tables[i]; i++)
@@ -195,7 +194,6 @@ flatpak_builtin_permission_list (int argc, char **argv,
     }
 
   flatpak_table_printer_print (printer);
-  flatpak_table_printer_free (printer);
 
   return TRUE;
 }
