@@ -1,4 +1,4 @@
-/*
+/* vi:set et sw=2 sts=2 cin cino=t0,f0,(0,{s,>2s,n-s,^-s,e-s:
  * Copyright © 2018 Red Hat, Inc
  *
  * This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@
 
 #include <glib/gi18n.h>
 
-#include "libglnx/libglnx.h"
+#include "libglnx.h"
 
 #include "flatpak-builtins.h"
 #include "flatpak-builtins-utils.h"
@@ -415,7 +415,7 @@ flatpak_builtin_repair (int argc, char **argv, GCancellable *cancellable, GError
 
        This does also mean that other areas of this code section that print errors will need to print a trailing
        newline as well, otherwise the output will overwrite any errors. */
-    if (flatpak_fancy_output ())
+    if (flatpak_fancy_output () && i != 1)
       g_print ("\033[A\r\033[K");
 
     g_print (_("[%d/%d] Verifying %s…\n"), i, g_hash_table_size (all_refs), refspec);
