@@ -101,7 +101,7 @@ flatpak_builtin_run (int argc, char **argv, GCancellable *cancellable, GError **
   g_autoptr(FlatpakDecomposed) runtime_ref = NULL;
   const char *pref;
   int i;
-  int rest_argv_start, rest_argc;
+  int rest_argv_start = 0, rest_argc = 0;
   g_autoptr(FlatpakContext) arg_context = NULL;
   g_autofree char *id = NULL;
   g_autofree char *arch = NULL;
@@ -114,7 +114,6 @@ flatpak_builtin_run (int argc, char **argv, GCancellable *cancellable, GError **
   context = g_option_context_new (_("APP [ARGUMENT…] - Run an app"));
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
 
-  rest_argc = 0;
   for (i = 1; i < argc; i++)
     {
       /* The non-option is the command, take it out of the arguments */
@@ -155,6 +154,8 @@ flatpak_builtin_run (int argc, char **argv, GCancellable *cancellable, GError **
   if (rest_argc == 0)
     return usage_error (context, _("APP must be specified"), error);
 
+  /* If we get here, then rest_argv_start must have been set >= 1 */
+  g_assert (rest_argv_start > 0);
   pref = argv[rest_argv_start];
 
   if (!flatpak_split_partial_ref_arg (pref, FLATPAK_KINDS_APP | FLATPAK_KINDS_RUNTIME,
