@@ -21,7 +21,7 @@
 #ifndef __FLATPAK_OCI_REGISTRY_H__
 #define __FLATPAK_OCI_REGISTRY_H__
 
-#include "libglnx/libglnx.h"
+#include "libglnx.h"
 
 #include <glib.h>
 #include <gio/gio.h>
@@ -43,14 +43,6 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC (FlatpakOciRegistry, g_object_unref)
 
 GType flatpak_oci_layer_writer_get_type (void);
 
-typedef enum {
-  FLATPAK_OCI_ERROR_NOT_CHANGED = 0,
-} FlatpakOciErrorEnum;
-
-#define FLATPAK_OCI_ERROR flatpak_oci_error_quark ()
-
-FLATPAK_EXTERN GQuark  flatpak_oci_error_quark (void);
-
 typedef struct FlatpakOciLayerWriter FlatpakOciLayerWriter;
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (FlatpakOciLayerWriter, g_object_unref)
@@ -75,6 +67,7 @@ int                    flatpak_oci_registry_download_blob (FlatpakOciRegistry
                                                            const char            *repository,
                                                            gboolean               manifest,
                                                            const char            *digest,
+                                                           const char           **alt_uris,
                                                            FlatpakLoadUriProgress progress_cb,
                                                            gpointer               user_data,
                                                            GCancellable          *cancellable,
@@ -89,6 +82,8 @@ GBytes             *   flatpak_oci_registry_load_blob (FlatpakOciRegistry *self,
                                                        const char         *repository,
                                                        gboolean            manifest,
                                                        const char         *digest,
+                                                       const char        **alt_uris,
+                                                       char              **out_content_type,
                                                        GCancellable       *cancellable,
                                                        GError            **error);
 char *                 flatpak_oci_registry_store_blob (FlatpakOciRegistry *self,
@@ -100,6 +95,7 @@ gboolean               flatpak_oci_registry_mirror_blob (FlatpakOciRegistry    *
                                                          const char            *repository,
                                                          gboolean               manifest,
                                                          const char            *digest,
+                                                         const char          **alt_uris,
                                                          FlatpakLoadUriProgress progress_cb,
                                                          gpointer               user_data,
                                                          GCancellable          *cancellable,
@@ -111,12 +107,14 @@ FlatpakOciDescriptor * flatpak_oci_registry_store_json (FlatpakOciRegistry *self
 FlatpakOciVersioned *  flatpak_oci_registry_load_versioned (FlatpakOciRegistry *self,
                                                             const char         *repository,
                                                             const char         *digest,
+                                                            const char        **alt_uris,
                                                             gsize              *out_size,
                                                             GCancellable       *cancellable,
                                                             GError            **error);
 FlatpakOciImage *      flatpak_oci_registry_load_image_config (FlatpakOciRegistry *self,
                                                                const char         *repository,
                                                                const char         *digest,
+                                                               const char        **alt_uris,
                                                                gsize              *out_size,
                                                                GCancellable       *cancellable,
                                                                GError            **error);
@@ -124,6 +122,22 @@ FlatpakOciLayerWriter *flatpak_oci_registry_write_layer (FlatpakOciRegistry *sel
                                                          GCancellable       *cancellable,
                                                          GError            **error);
 
+int                     flatpak_oci_registry_apply_delta (FlatpakOciRegistry    *self,
+                                                          int                    delta_fd,
+                                                          GFile                 *content_dir,
+                                                          GCancellable          *cancellable,
+                                                          GError               **error);
+char *                  flatpak_oci_registry_apply_delta_to_blob (FlatpakOciRegistry    *self,
+                                                                  int                    delta_fd,
+                                                                  GFile                 *content_dir,
+                                                                  GCancellable          *cancellable,
+                                                                  GError               **error);
+FlatpakOciManifest *   flatpak_oci_registry_find_delta_manifest (FlatpakOciRegistry    *registry,
+                                                                 const char            *oci_repository,
+                                                                 const char            *for_digest,
+                                                                 const char            *delta_manifest_uri,
+                                                                 GCancellable          *cancellable);
+
 struct archive *flatpak_oci_layer_writer_get_archive (FlatpakOciLayerWriter *self);
 gboolean        flatpak_oci_layer_writer_close (FlatpakOciLayerWriter *self,
                                                 char                 **uncompressed_digest_out,
@@ -146,24 +160,24 @@ FlatpakOciSignature *flatpak_oci_verify_signature (OstreeRepo *repo,
                                                    GBytes     *signature,
                                                    GError    **error);
 
-gboolean flatpak_oci_index_ensure_cached (SoupSession  *soup_session,
-                                          const char   *uri,
-                                          GFile        *index,
-                                          char        **index_uri_out,
-                                          GCancellable *cancellable,
-                                          GError      **error);
+gboolean flatpak_oci_index_ensure_cached (FlatpakHttpSession  *http_session,
+                                          const char          *uri,
+                                          GFile               *index,
+                                          char               **index_uri_out,
+                                          GCancellable        *cancellable,
+                                          GError             **error);
 
 GVariant *flatpak_oci_index_make_summary (GFile        *index,
                                           const char   *index_uri,
                                           GCancellable *cancellable,
                                           GError      **error);
 
-GBytes *flatpak_oci_index_make_appstream (SoupSession  *soup_session,
-                                          GFile        *index,
-                                          const char   *index_uri,
-                                          const char   *arch,
-                                          int           icons_dfd,
-                                          GCancellable *cancellable,
-                                          GError      **error);
+GBytes *flatpak_oci_index_make_appstream (FlatpakHttpSession  *http_session,
+                                          GFile               *index,
+                                          const char          *index_uri,
+                                          const char          *arch,
+                                          int                  icons_dfd,
+                                          GCancellable        *cancellable,
+                                          GError             **error);
 
 #endif /* __FLATPAK_OCI_REGISTRY_H__ */
