@@ -43,7 +43,7 @@ char * flatpak_make_valid_id_prefix (const char *orig_id);
 gboolean flatpak_id_has_subref_suffix (const char *id,
                                        gssize      id_len);
 
-gboolean is_flatpak_ref (const char *ref);
+gboolean flatpak_is_app_runtime_or_appstream_ref (const char *ref);
 char * flatpak_get_arch_for_ref (const char *ref);
 const char *flatpak_get_compat_arch_reverse (const char *compat_arch);
 
@@ -88,6 +88,8 @@ gboolean           flatpak_decomposed_equal                 (FlatpakDecomposed
                                                              FlatpakDecomposed  *ref_b);
 gboolean           flatpak_decomposed_equal_except_branch   (FlatpakDecomposed  *ref_a,
                                                              FlatpakDecomposed  *ref_b);
+gboolean           flatpak_decomposed_equal_except_arch     (FlatpakDecomposed  *ref_a,
+                                                             FlatpakDecomposed  *ref_b);
 gint               flatpak_decomposed_strcmp                (FlatpakDecomposed  *ref_a,
                                                              FlatpakDecomposed  *ref_b);
 gint               flatpak_decomposed_strcmp_p              (FlatpakDecomposed **ref_a,
