@@ -104,23 +104,25 @@ function parse_test {
     fi
 }
 
-for test in $1; do
+for test in $2; do
     parse_test $test
 done
 
-echo "# This file is autogenerated by make update-test-matrix, don't edit"
-echo "TEST_MATRIX= \\"
-for f in $tests; do
-    echo "	${f} \\"
-done
-echo "	\$(NULL)"
-echo "TEST_MATRIX_DIST= \\"
-for f in $dist_tests; do
-    echo "	${f} \\"
-done
-echo "	\$(NULL)"
-echo "TEST_MATRIX_EXTRA_DIST= \\"
-for f in $extra_dist_tests; do
-    echo "	${f} \\"
-done
-echo "	\$(NULL)"
+if [ "$1" = "--automake" ]; then
+    echo "# This file is autogenerated by ./tests/update-test-matrix, don't edit"
+    echo "TEST_MATRIX= \\"
+    for f in $tests; do
+        echo "	${f} \\"
+    done
+    echo "	\$(NULL)"
+    echo "TEST_MATRIX_DIST= \\"
+    for f in $dist_tests; do
+        echo "	${f} \\"
+    done
+    echo "	\$(NULL)"
+    echo "TEST_MATRIX_EXTRA_DIST= \\"
+    for f in $extra_dist_tests; do
+        echo "	${f} \\"
+    done
+    echo "	\$(NULL)"
+fi
